/*
 * JBoss, Home of Professional Open Source
 * Copyright 2007, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags.
 * See the copyright.txt in the distribution for a full listing
 * of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License, v. 2.0.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License,
 * v. 2.0 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2007,
 * @author JBoss Inc.
 */
package org.jboss.txbridge.ba.annotation;

import java.lang.annotation.*;

/**
 * Specifies all information related to the datamgmt action for a single service (method).
 * Following information is required:
 * - name of the datamgmt action
 * - location of the datamgmt action
 * - should the datamgmt action be executed only once in a BA
 * - what data should be remembered so that datamgmt can be executed
 * - what is the mode of datamgmt (is datamgmt action present as a method of an EJB,
 *   POJO or a Web Service)
 *
 * All set member values of this annotation override those which were set using the @BAService
 * annotation.
 *
 * @author Maciej P. Machulak (mmachulak@redhat.com)
 * @version 0.1
 */
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface BACompletedBy
{
    public String value();
    public Class[] serviceClass() default {};
    public Class[] ejbInterface() default {};
    public String jndiName() default "";
    public String providerURL() default "";
    public String wsdl() default "";
    public String namespace() default "";
    public String serviceName() default "";
    public boolean single() default false;
    public ParameterMatch match() default ParameterMatch.STRICT;
    public ExecutionOrder order() default ExecutionOrder.NORMAL;
    public DataMatch type() default DataMatch.RETURN_VALUE;
    public ExecutionMode mode() default ExecutionMode.EJB;
}
